use std::collections::HashMap;

use crate::utils::input_reader;
use crate::year_2023::day_08_haunted_wasteland::common::{parse_line, Node};

pub fn part_two() -> String {
    if let Ok(lines) = input_reader::read_lines("./input/2023/day_08/input_test_part_2.txt") {
        let mut node_map: HashMap<String, &Node> = HashMap::new();
        let mut start_nodes: Vec<&Node> = Vec::new();
        let mut instructions: Vec<char> = Vec::new();
        let mut s: String;

        for (i, line) in lines.enumerate() {
            s = line.unwrap();

            if i == 0 {
                instructions = s.chars().collect();
                continue;
            }

            if s.is_empty() {
                continue;
            }

            let node = parse_line(&s);
            let value = node.value.clone();

            node_map.insert(value.clone(), &node.clone());

            if value.ends_with('A') {
                let cloned_node = node.clone();
                start_nodes.push(&cloned_node);
            }
        }

        let mut curr_nodes: Vec<&Node> = start_nodes;
        let mut count = 0;
        let mut cond = true;

        while cond {
            let i = count % instructions.len();
            let c = instructions[i];

            let mut new_curr: Vec<&Node> = Vec::new();
            cond = false;

            for n in curr_nodes {
                let nn: &Node;
                if instructions[i] == 'L' {
                    nn = &node_map[&n.left];
                } else if instructions[i] == 'R' {
                    nn = &node_map[&n.right];
                } else {
                    panic!("Unknown instruction!")
                }

                if !nn.value.clone().ends_with('Z') {
                    cond = true;
                }

                new_curr.push(nn);
            }

            curr_nodes = new_curr;
            count += 1;
        }

        return count.to_string();
    } else {
        panic!("Failed to read lines from input file");
    }
}
